Package: ctmcd
Type: Package
Title: Estimating the Parameters of a Continuous-Time Markov Chain from
        Discrete-Time Data
Version: 1.4.4
Date: 2024-02-07
Authors@R: c(person("Marius", "Pfeuffer", email = "marius.pfeuffer@fau.de", role = c("aut", "cre")),
    person("Greig", "Smith", role = "ctb"),
    person("Goncalo", "dos Reis", role = "ctb"),
    person("Linda", "Moestel", role = "ctb"),
    person("Matthias", "Fischer", role = "ctb"))
Description: Estimation of Markov generator matrices from discrete-time observations. The implemented approaches comprise diagonal and weighted adjustment of matrix logarithm based candidate solutions as in Israel (2001) <doi:10.1111/1467-9965.00114> as well as a quasi-optimization approach. Moreover, the expectation-maximization algorithm and the Gibbs sampling approach of Bladt and Sorensen (2005) <doi:10.1111/j.1467-9868.2005.00508.x> are included.
License: GPL-3
Imports: Rcpp (>= 1.0.12), coda, expm, numDeriv
Suggests: knitr, rmarkdown, R.rsp
LinkingTo: Rcpp, RcppArmadillo
VignetteBuilder: knitr, R.rsp
NeedsCompilation: yes
Packaged: 2024-02-08 18:35:26 UTC; Stefan
Author: Marius Pfeuffer [aut, cre],
  Greig Smith [ctb],
  Goncalo dos Reis [ctb],
  Linda Moestel [ctb],
  Matthias Fischer [ctb]
Maintainer: Marius Pfeuffer <marius.pfeuffer@fau.de>
Repository: CRAN
Date/Publication: 2024-02-09 09:40:02 UTC
Built: R 4.2.3; aarch64-apple-darwin20; 2024-02-09 10:22:53 UTC; unix
Archs: ctmcd.so.dSYM

Package: ctmcd
Type: Package
Title: Estimating the Parameters of a Continuous-Time Markov Chain from
        Discrete-Time Data
Version: 1.4.4
Date: 2024-02-07
Authors@R: c(person("Marius", "Pfeuffer", email = "marius.pfeuffer@fau.de", role = c("aut", "cre")),
    person("Greig", "Smith", role = "ctb"),
    person("Goncalo", "dos Reis", role = "ctb"),
    person("Linda", "Moestel", role = "ctb"),
    person("Matthias", "Fischer", role = "ctb"))
Description: Estimation of Markov generator matrices from discrete-time observations. The implemented approaches comprise diagonal and weighted adjustment of matrix logarithm based candidate solutions as in Israel (2001) <doi:10.1111/1467-9965.00114> as well as a quasi-optimization approach. Moreover, the expectation-maximization algorithm and the Gibbs sampling approach of Bladt and Sorensen (2005) <doi:10.1111/j.1467-9868.2005.00508.x> are included.
License: GPL-3
Imports: Rcpp (>= 1.0.12), coda, expm, numDeriv
Suggests: knitr, rmarkdown, R.rsp
LinkingTo: Rcpp, RcppArmadillo
VignetteBuilder: knitr, R.rsp
NeedsCompilation: yes
Packaged: 2024-02-08 18:35:26 UTC; Stefan
Author: Marius Pfeuffer [aut, cre],
  Greig Smith [ctb],
  Goncalo dos Reis [ctb],
  Linda Moestel [ctb],
  Matthias Fischer [ctb]
Maintainer: Marius Pfeuffer <marius.pfeuffer@fau.de>
Repository: CRAN
Date/Publication: 2024-02-09 09:40:02 UTC

useDynLib(ctmcd, .registration=TRUE)
importFrom(Rcpp, evalCpp)
import(stats)
import(coda)
import(expm)
import(graphics)
import(grDevices)
import(utils)
import(numDeriv)
export(gmDA)
export(gmWA)
export(gmQO)
export(gmEM)
export(gmGS)
export(print.gm)
export(plot.gm)
export(summary.gm)
export(gm.default)
export(rNijTRiT_ModRej)
export(rNijTRiT_Unif)
export(ciEMBS)
export(ciEMSdR)
export(ciEMoFI)
export(tmci)
export(ciGS)
export(plotM)
export(gm)
export(ctmcdlogLik)
export(gmci)
export(gmci.default)
export(print.gmci)
export(plot.gmci)
S3method(gm,default)
S3method(plot,gm)
S3method(print,gm)
S3method(summary,gm)
S3method(gmci,default)
S3method(plot,gmci)
S3method(print,gmci)
exportPattern("^[[:alpha:]]+")

plotM <-
function (mat, mattext, col = c("grey", "red"), main, las = 1, 
    xlab = "To", ylab = "From", xnames, ynames, cex = min(1, 
        nrow(mat)/8), fig = 3, opacity_factor) 
{
    mat = as.matrix(mat)
    if (missing(main)) {
        main = ""
    }
    if (missing(mattext)) {
        mattext = round(mat, fig)
    }
    if (missing(xnames)) {
        xnames = dimnames(mat)[[2]]
    }
    if (missing(ynames)) {
        ynames = dimnames(mat)[[1]]
    }
    nc = ncol(mat)
    nr = nrow(mat)
    posmat = mat
    posmat[which(posmat <= 0)] = NA
    negmat = mat
    negmat[which(mat >= 0)] = NA
    if (missing(opacity_factor)) {
        opacity_factor = vector(length = 2)
        if (prod(is.na(posmat)) == 0) {
            opacity_factor[1] = max(posmat[which(posmat > 0)])/quantile(posmat[which(posmat > 
                0)], 0.75)[[1]]
        }
        if (prod(is.na(negmat)) == 0) {
            opacity_factor[2] = max(abs(negmat)[which(negmat < 
                0)])/quantile(abs(negmat)[which(negmat < 0)], 
                0.75)[[1]]
        }
    }
    specp = rev(1 - ((0:(nr * nc))/(nr * nc))^opacity_factor[1])
    specn = rev(1 - ((0:(nr * nc))/(nr * nc))^opacity_factor[2])
    if (prod(is.na(posmat)) == 0) {
        image(t(apply(posmat, 2, rev)), col = rgb(t(col2rgb(col[1]))/255, 
            alpha = specp), main = main, axes = F, zlim = c(0, 
            max(mat)), xlab = xlab, ylab = ylab)
    }
    if (prod(is.na(negmat)) == 0) {
        image(t(apply(abs(negmat), 2, rev)), col = rgb(t(col2rgb(col[2]))/255, 
            alpha = specn), main = main, axes = F, zlim = c(0, 
            abs(min(mat))), xlab = xlab, ylab = ylab, add = T)
    }
    axis(1, (0:(nc - 1))/(nc - 1), xnames, las = las)
    axis(2, (0:(nr - 1))/(nr - 1), rev(ynames), las = las)
    rvec = (0:(nr - 1)/(nr - 1))
    cvec = (0:(nc - 1)/(nc - 1))
    for (j in 1:nr) {
        text(cvec, 1 - rvec[j], mattext[j, ], cex = cex)
    }
}

summary.gm <-
function (object, ...) 
{
    cat("Call:\n")
    print(object$call)
    cat("\nMethod:\n")
    print(object$method)
    cat("\nParameters:\n")
    print(signif(object$par, 4))
    if (!is.null(object$burnin)) {
        cat("\nBurn-in:\n")
        print(object$burnin)
    }
    if (!is.null(object$niter)) {
        cat("\nNumber of Iterations:\n")
        print(object$niter)
    }
}

print.gmci <-
function (x, ...) 
{
    cat("Call:\n")
    print(x$call)
    cat("\nMethod:\n")
    print(x$method)
    cat("\nLower Limit:\n")
    print(signif(x$lower, 4))
    cat("\nUpper Limit:\n")
    print(signif(x$upper, 4))
}

ciGS <-
function (x, alpha) 
{
    draws = length(x$draws)
    n = nrow(x$draws[[1]])
    draws = length(x$draws)
    data = matrix(unlist(x$draws), n^2, draws)
    lowermat = matrix(0, n, n)
    uppermat = matrix(0, n, n)
    for (row in 1:n) {
        for (col in 1:n) {
            lowermat[row, col] = quantile(data[(col - 1) * n + 
                row, ], alpha/2)
            uppermat[row, col] = quantile(data[(col - 1) * n + 
                row, ], 1 - alpha/2)
        }
    }
    limits = list(lower = lowermat, upper = uppermat)
    limits
}

gmWA <-
function (tmrel, te, logmethod = "Eigen") 
{
    n = nrow(tmrel)
    gmest = logm(tmrel, method = logmethod)/te
    for (i in 1:(n - 1)) {
        gmiNeg = -sum((gmest * (gmest < 0))[i, setdiff(1:n, i)])
        gmiPos = sum((gmest * (gmest > 0))[i, ])
        gmest[i, setdiff(1:n, i)] = gmest[i, setdiff(1:n, i)] - 
            gmiNeg/gmiPos * abs(gmest[i, setdiff(1:n, i)])
    }
    gmest[setdiff(which(gmest < 0), seq(1, n^2, n + 1))] = 0
    return(gmest)
}

ciEMBS <-
function (x, alpha, eps = 1e-04, expmethod = "PadeRBS") 
{
    tmabs = x$tm
    te = x$te
    ERiT_func = function(x, Q, i0, j0, i, j, ...) {
        n = nrow(Q)
        Q[i0, j0] = x
        diag(Q) = 0
        diag(Q) = -rowSums(Q)
        dtt = expm(Q * te, method = expmethod)
        uitui = matrix(0, n, n)
        uitui[i, i] = 1
        augmat = rbind(cbind(Q, uitui), cbind(matrix(0, n, n), 
            Q))
        caseRiT = expm(te * augmat, method = expmethod)[1:n, 
            (n + 1):(2 * n)]
        ERiT = sum(tmabs * caseRiT/dtt, na.rm = TRUE)
        ERiT
    }
    ENijT_func = function(x, Q, i0, j0, i, j, ...) {
        n = nrow(Q)
        Q[i0, j0] = x
        diag(Q) = 0
        diag(Q) = -rowSums(Q)
        dtt = expm(Q * te, method = expmethod)
        uituj = matrix(0, n, n)
        uituj[i, j] = 1
        augmat = rbind(cbind(Q, uituj), cbind(matrix(0, n, n), 
            Q))
        caseNijT = Q[i, j] * expm(te * augmat, method = expmethod)[1:n, 
            (n + 1):(2 * n)]
        ENijT = sum(tmabs * caseNijT/dtt, na.rm = TRUE)
        ENijT
    }
    ParMat = x$par
    ParMat[which(x$par < eps)] = 0
    npar = sum(ParMat > 0)
    ivec = as.vector(row(ParMat) * (ParMat != 0))
    jvec = as.vector(col(ParMat) * (ParMat != 0))
    ivec = ivec[which(ivec != 0)]
    jvec = jvec[which(jvec != 0)]
    diag(ParMat) = -rowSums(ParMat)
    FI = matrix(0, length(ivec), length(jvec))
    for (k in 1:length(ivec)) {
        for (l in 1:length(ivec)) {
            if (l == k) {
                ERiTder = grad(ERiT_func, x = x$par[ivec[k], 
                  jvec[k]], Q = x$par, i0 = ivec[k], j0 = jvec[k], 
                  i = ivec[k], j = jvec[k])
                ENijTder = grad(ENijT_func, x = x$par[ivec[k], 
                  jvec[k]], Q = x$par, i0 = ivec[k], j0 = jvec[k], 
                  i = ivec[k], j = jvec[k])
                FI[k, k] = 1/x$par[ivec[k], jvec[k]]^2 * x$ENijT[ivec[k], 
                  jvec[k]] - 1/x$par[ivec[k], jvec[k]] * ENijTder + 
                  ERiTder
            }
            else {
                ERiTder = grad(ERiT_func, x = x$par[ivec[l], 
                  jvec[l]], Q = x$par, i0 = ivec[l], j0 = jvec[l], 
                  i = ivec[k], j = jvec[k])
                ENijTder = grad(ENijT_func, x = x$par[ivec[l], 
                  jvec[l]], Q = x$par, i0 = ivec[l], j0 = jvec[l], 
                  i = ivec[k], j = jvec[k])
                FI[k, l] = -1/x$par[ivec[k], jvec[k]] * ENijTder + 
                  ERiTder
            }
        }
    }
    CVmat = solve((FI + t(FI))/2)
    SEvec = sqrt(diag(CVmat))
    SEmat = matrix(0, nrow(tmabs), nrow(tmabs))
    for (k in 1:length(ivec)) {
        SEmat[ivec[k], jvec[k]] = SEvec[k]
    }
    diagse = vector(length = nrow(tmabs))
    for (i in unique(ivec)) {
        elem = jvec[which(ivec == i)]
        if (length(elem) == 1) {
            diagse[i] = SEmat[i, elem]
        }
        else {
            combs = combn(elem, 2)
            CVsum = 0
            for (k in 1:ncol(combs)) {
                par1 = intersect(which(ivec == i), which(jvec == 
                  combs[1, k]))
                par2 = intersect(which(ivec == i), which(jvec == 
                  combs[2, k]))
                CVsum = CVsum + CVmat[par1, par2]
            }
            diagse[i] = sqrt(sum(SEmat[i, elem]^2) + 2 * CVsum)
        }
    }
    diag(SEmat) = diagse
    lowermat = x$par - qnorm(1 - alpha/2) * SEmat
    lowermat[which(SEmat == 0)] = NA
    lowermat[which(diag(x$par) == 0), ] = 0
    uppermat = x$par + qnorm(1 - alpha/2) * SEmat
    uppermat[which(SEmat == 0)] = NA
    uppermat[which(diag(x$par) == 0), ] = 0
    limits = list(lower = lowermat, upper = uppermat)
    limits
}

ciEMoFI=function (x, alpha, eps = 1e-04, expmethod = "PadeRBS") 
{
  JVector = function(h, i) {
    J = rep(0, h)
    J[i] = 1
    J
  }
  JMatrix = function(h, i, j) {
    J = matrix(0, h, h)
    J[i, j] = 1
    J
  }
  signif_level = alpha
  Q = x$par
  h = nrow(Q)
  te = x$te
  tmabs = x$tm
  QUseable = matrix(0, h, h)
  for (i in 1:h) {
    for (j in 1:h) {
      if (Q[i, j] > eps && j != i) 
        QUseable[i, j] = 1
    }
  }
  VQ = matrix(0, sum(sum(QUseable)), 2)
  Points = length(VQ[, 1])
  counter = 1
  for (i in 1:h) {
    for (j in 1:h) {
      if (QUseable[i, j] == 1) {
        VQ[counter, 1] = i
        VQ[counter, 2] = j
        counter = counter + 1
      }
    }
  }
  Hess = matrix(0, Points, Points)
  LowerPart1 = matrix(0, h, h)
  LowerPart2 = matrix(0, 2 * h, 2 * h)
  ExpQ = expm(Q * te, method = expmethod)
  Ceta=list()
  MatrixExponentialeta=list()
  for(i in 1:Points){
    alpha = VQ[i, 1]
    beta = VQ[i, 2]
    UpperParteta = JVector(h, alpha) %*% t(JVector(h,beta)) - JVector(h, alpha) %*% t(JVector(h, alpha))
    Ceta[[i]] = rbind(cbind(Q, UpperParteta), cbind(LowerPart1, Q))
    MatrixExponentialeta[[i]] = expm(Ceta[[i]] * te, method = expmethod)
  }
  for (i in 1:Points) {
    alpha = VQ[i, 1]
    beta = VQ[i, 2]
    for (j in 1:Points) {
      mu = VQ[j, 1]
      nu = VQ[j, 2]
      UpperPartxi = rbind(cbind(JVector(h, mu) %*% t(JVector(h, nu)) - JVector(h, mu) %*% t(JVector(h,mu)),
                                LowerPart1), cbind(LowerPart1, JVector(h,mu) %*% t(JVector(h, nu)) - JVector(h, mu) %*% t(JVector(h, mu))))
      Cxi = rbind(cbind(Ceta[[i]], UpperPartxi), cbind(LowerPart2, Ceta[[i]]))
      MatrixExponentialxi = expm(Cxi * te, method = expmethod)
      for (s in 1:h) {
        for (r in 1:h) {
          if (tmabs[s, r] > 0) {
            Hess[i, j] = Hess[i, j] + tmabs[s, r] * ExpQ[s, r]^(-1) *( MatrixExponentialxi[s,r+3*h]
                                                                       -ExpQ[s, r]^(-1) * MatrixExponentialeta[[i]][s, h+ r])*MatrixExponentialeta[[j]][s,h+r]
          }
        }
      }
    }
  }
  CVmat = -solve((Hess + t(Hess))/2)
  SEvec = sqrt(diag(CVmat))
  SEmat = matrix(0, nrow(tmabs), nrow(tmabs))
  for (k in 1:length(VQ[, 1])) {
    SEmat[VQ[k, 1], VQ[k, 2]] = SEvec[k]
  }
  diagse = vector(length = nrow(tmabs))
  for (i in unique(VQ[, 1])) {
    elem = VQ[which(VQ[, 1] == i), 2]
    if (length(elem) == 1) {
      diagse[i] = SEmat[i, elem]
    }
    else {
      combs = combn(elem, 2)
      CVsum = 0
      for (k in 1:ncol(combs)) {
        par1 = intersect(which(VQ[, 1] == i), which(VQ[,2] == combs[1, k]))
        par2 = intersect(which(VQ[, 1] == i), which(VQ[,2] == combs[2, k]))
        CVsum = CVsum + CVmat[par1, par2]
      }
      diagse[i] = sqrt(sum(SEmat[i, elem]^2) + 2 * CVsum)
    }
  }
  diag(SEmat) = diagse
  lowermat = Q - qnorm(1 - signif_level/2) * SEmat
  lowermat[which(SEmat == 0)] = NA
  lowermat[which(diag(Q) == 0), ] = 0
  uppermat = Q + qnorm(1 - signif_level/2) * SEmat
  uppermat[which(SEmat == 0)] = NA
  uppermat[which(diag(Q) == 0), ] = 0
  limits = list(lower = lowermat, upper = uppermat, FI=Hess, CVmat=CVmat)
  limits
}

gmQO <-
function (tmrel, te, logmethod = "Eigen") 
{
    n = nrow(tmrel)
    gmest = logm(tmrel, method = logmethod)/te
    for (i in 1:n) {
        a = gmest[i, ]
        lambda = mean(a)
        aorder = order(a + lambda)
        aest = a[aorder]
        for (m in 2:n) {
            if ((n - m + 1) * aest[m + 1] - (aest[1] + sum(aest[(m + 
                1):n])) >= 0) {
                mstar = m
                break
            }
        }
        zstar = NULL
        for (j in 1:n) {
            if (j %in% 2:mstar) {
                zstar = c(zstar, 0)
            }
            else {
                zstar = c(zstar, aest[j] - 1/(n - mstar + 1) * 
                  (aest[1] + sum(aest[(mstar + 1):n])))
            }
        }
        gmest[i, ] = zstar[order(aorder)]
    }
    return(gmest)
}

plot.gmci <-
function (x, mattext, col = c("grey", "red"), main, las = 1, 
    xlab = "To", ylab = "From", xnames, ynames, cex = 1, fig = 2, 
    opacity_factor, ...) 
{
    if (missing(main)) {
        main = paste0(round(100 * (1 - x$alpha), 3), "% ", x$method)
    }
    n = nrow(x$par)
    if (missing(mattext)) {
        mattext = matrix(paste0("[", signif(x$lower, fig), "; ", 
            signif(x$upper, fig), "]"), n, n)
        mattext[which(is.na(x$lower) & is.na(x$upper))] = ""
    }
    plotM(x$par, mattext, col, main, las, xlab, ylab, xnames, 
        ynames, cex, fig, opacity_factor)
}

plot.gm <-
function (x, mattext, col = c("grey", "red"), main = x$method, 
    las = 1, xlab = "To", ylab = "From", xnames, ynames, cex = 1, 
    fig = 3, opacity_factor, ...) 
{
    plotM(x$par, mattext, col, main, las, xlab, ylab, xnames, 
        ynames, cex, fig, opacity_factor)
}

ctmcdlogLik <-
function (gm, tmabs, te) 
{
    P = expm(gm * te)
    ll = 0
    for (i in 1:nrow(P)) {
        for (j in 1:ncol(P)) {
            if (P[i, j] > 0) {
                ll = ll + tmabs[i, j] * log(P[i, j])
            }
        }
    }
    return(ll)
}

gmci.default <-
  function (gm, alpha, eps = 1e-04, cimethod= "Direct", expmethod = "PadeRBS", ...) 
  {
    if (gm$method == "Expectation-Maximization Algorithm") {
      if(cimethod=="SdR"){
        limits = ciEMSdR(gm, alpha, eps, expmethod)
      }
      if(cimethod=="BS"){
        limits = ciEMBS(gm, alpha, eps, expmethod)
      }
      if(cimethod=="Direct"){
        limits = ciEMoFI(gm, alpha, eps, expmethod)
      }
      limits$method = "Wald Confidence Interval"
    }
    if (gm$method == "Gibbs Sampler") {
      limits = ciGS(gm, alpha)
      limits$method = "Equal Tailed Credibility Interval"
    }
    limits$par = gm$par
    limits$alpha = alpha
    limits$call = match.call()
    class(limits) = "gmci"
    limits
  }

gm.default <-
function (tm, te, method, gmguess = NULL, prior = NULL, burnin = NULL, 
    eps = 1e-06, conv_pvalue = 0.05, conv_freq = 10, niter = 10000, 
    sampl_func = NULL, combmat = NULL, sampl_method = "Unif", 
    logmethod = "Eigen", expmethod = "PadeRBS", verbose = FALSE, 
    ...) 
{
    tm = as.matrix(tm)
    te = as.numeric(te)
    est = list()
    if (method == "DA") {
        est$par = gmDA(tmrel = tm, te, logmethod = logmethod)
        est$method = "Diagonal Adjustment"
    }
    else if (method == "WA") {
        est$par = gmWA(tmrel = tm, te, logmethod = logmethod)
        est$method = "Weighted Adjustment"
    }
    else if (method == "QO") {
        est$par = gmQO(tmrel = tm, te, logmethod = logmethod)
        est$method = "Quasi Optimization"
    }
    else if (method == "EM") {
        est = gmEM(tmabs = tm, te, gmguess, eps, niter, expmethod, 
            verbose)
        est$method = "Expectation-Maximization Algorithm"
    }
    else if (method == "GS") {
        est = gmGS(tmabs = tm, te, prior, burnin, conv_pvalue, 
            conv_freq, niter, sampl_method, expmethod, verbose, 
            combmat, sampl_func)
        est$method = "Gibbs Sampler"
    }
    est$call = match.call()
    est$tm = tm
    est$te = te
    class(est) = "gm"
    est
}

gmEM <-
function (tmabs, te, gmguess, eps = 1e-06, niter = 10000, expmethod = "PadeRBS", 
    verbose = FALSE) 
{
    n = ncol(tmabs)
    if (nrow(tmabs) == n - 1) {
        tmabs = rbind(tmabs, rep(0, n))
    }
    llvec = NULL
    gmprev = gmguess
    for (k in 1:niter) {
        dtt = expm(gmprev * te, method = expmethod)
        ERiT = rep(0, n)
        for (i in 1:n) {
            uitui = matrix(0, n, n)
            uitui[i, i] = 1
            augmat = rbind(cbind(gmprev, uitui), cbind(matrix(0, 
                n, n), gmprev))
            caseRiT = expm(te * augmat, method = expmethod)[1:n, 
                (n + 1):(2 * n)]
            ERiT[i] = ERiT[i] + sum(tmabs * caseRiT/dtt, na.rm = TRUE)
        }
        ENijT = matrix(0, n, n)
        for (i in 1:n) {
            for (j in setdiff(1:n, i)) {
                uituj = matrix(0, n, n)
                uituj[i, j] = 1
                augmat = rbind(cbind(gmprev, uituj), cbind(matrix(0, 
                  n, n), gmprev))
                caseNijT = gmprev[i, j] * expm(te * augmat, method = expmethod)[1:n, 
                  (n + 1):(2 * n)]
                ENijT[i, j] = ENijT[i, j] + sum(tmabs * caseNijT/dtt, 
                  na.rm = TRUE)
            }
        }
        gmest = rbind(ENijT/ERiT)
        diag(gmest) = -rowSums(gmest)
        gmprev = gmest
        ll = ctmcdlogLik(gmest, tmabs, te)
        llvec = c(llvec, ll)
        rel_err = abs(llvec[length(llvec)] - llvec[length(llvec) - 
            1])/abs(llvec[length(llvec) - 1])
        if (verbose == TRUE) {
            print(paste0("Iteration: ", k, "  Log-Likelihood: ", 
                signif(ll, 6)))
        }
        if (k > 2 && rel_err <= eps) {
            break
        }
    }
    colnames(gmest) = colnames(tmabs)
    rownames(gmest) = rownames(tmabs)
    est = list(par = gmest, niter = k, eps = rel_err, ll = llvec, 
        ENijT = ENijT, ERiT = ERiT)
    return(est)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

rNijTRiT_ModRej <- function(tmabs, te, gm) {
    .Call(`_ctmcd_rNijTRiT_ModRej`, tmabs, te, gm)
}

rNijTRiT_Unif <- function(tmabs, te, gm, tpm) {
    .Call(`_ctmcd_rNijTRiT_Unif`, tmabs, te, gm, tpm)
}

# Register entry points for exported C++ functions
methods::setLoadAction(function(ns) {
    .Call(`_ctmcd_RcppExport_registerCCallable`)
})

expmMC =
  function(gm, t, method="PadeRBS",order=8) {
    if (!requireNamespace("markovchain", quietly = TRUE)) {
      stop("markovchain is needed for this function to work. Please install it.",
           call. = FALSE)
    }
    if(!(class(gm) %in% c("gm","matrix","Matrix")))
      stop("Error! Expecting either a matrix or a gm object")
    if ( class(gm) %in% c("matrix","Matrix")) generator_matrix = gm else generator_matrix = as.matrix(gm[["par"]])
    transitionMatrix = expm(generator_matrix*t,method=method,order=order)
    out = list()
    attr(out,"states") = letters[1:nrow(transitionMatrix)]
    attr(out,"byrow") = TRUE
    rownames(transitionMatrix) = letters[1:nrow(transitionMatrix)]
    colnames(transitionMatrix) = letters[1:ncol(transitionMatrix)]
    attr(out,"transitionMatrix") = transitionMatrix
    attr(out,"name") = "Unnamed Markov chain"
    class(out) = "markovchain"
    attr(attr(out,"class"),"package") = "markovchain"
    out = asS4(out)
    return(out)
  }

ciEMSdR=function(x,alpha,eps=1e-04,expmethod="PadeRBS"){
  JVector = function(h,i){
    J = rep(0,h)
    J[i] = 1
    J
  }
  
  JMatrix = function(h,i,j){
    J = matrix(0,h,h)
    J[i,j] = 1
    J
  }
  
  signif_level = alpha
  Q = x$par
  h = nrow(Q)
  te = x$te
  tmabs = x$tm
  
  QUseable = matrix(0,h,h)
  
  for(i in 1:h){
    for(j in 1:h){
      if(Q[i,j]>eps && j!=i)  QUseable[i,j] = 1
    }
  }
  
  VQ = matrix(0,sum(sum(QUseable)),2) 
  Points = length(VQ[,1])
  
  counter = 1
  for(i in 1:h){
    for(j in 1:h){
      if(QUseable[i,j]==1){
        VQ[counter,1] = i
        VQ[counter,2] = j
        counter = counter+1
      }
    }
  }
  
  Q = Q*(Q>eps)
  diag(Q) = -rowSums(Q)
  
  Hess = matrix(0,Points,Points)
  
  LowerPart1 = matrix(0,h,h)
  LowerPart2 = matrix(0,2*h,2*h)
  
  for(i in 1:Points){
    for(j in 1:Points){
      alpha = VQ[i,1]
      beta = VQ[i,2]
      mu = VQ[j,1]
      nu = VQ[j,2]
      
      UpperPartgamma = Q[mu,nu]*JVector(h,mu)%*%t(JVector(h,nu))
      
      #Build the matrix to be exponentiated.
      Cgamma = rbind(cbind(Q,UpperPartgamma),cbind(LowerPart1,Q))
      MatrixExponentialgamma = expm(Cgamma*te,method=expmethod)
      
      #eta
      UpperParteta = JVector(h,alpha)%*%t(JVector(h,beta))-JVector(h,alpha)%*%t(JVector(h,alpha)) #JVector is a row vector
      #Build the matrix to be exponentiated.
      Ceta = rbind(cbind(Q,UpperParteta),cbind(LowerPart1,Q))
      MatrixExponentialeta = expm(Ceta*te,method=expmethod)
      
      #phi
      UpperPartphi = JVector(h,mu)%*%t(JVector(h,mu)) #JVector is a row vector
      #Build the matrix to be exponentiated.
      Cphi = rbind(cbind(Q,UpperPartphi),cbind(LowerPart1, Q))
      MatrixExponentialphi = expm(Cphi*te,method=expmethod)
      
      #Let us now calculate the bigger matrices
      
      #psi
      #derivative of Cgamma
      DCgammaUpper = JVector(h,mu)%*%t(JVector(h,nu))*(mu==alpha && nu==beta)
      UpperPartpsi = rbind(cbind(JVector(h,alpha)%*%t(JVector(h,beta))-JVector(h,alpha)%*%t(JVector(h,alpha)),DCgammaUpper),
                         cbind(LowerPart1, JVector(h,alpha)%*%t(JVector(h,beta))-JVector(h,alpha)%*%t(JVector(h,alpha))))
      
      #Build the matrix to be exponentiated.
      Cpsi = rbind(cbind(Cgamma,UpperPartpsi),cbind(LowerPart2, Cgamma))
      MatrixExponentialpsi = expm(Cpsi*te,method=expmethod)
      
      #omega
      #derivative of Cphi
      UpperPartomega = rbind(cbind(JVector(h,alpha)%*%t(JVector(h,beta))-JVector(h,alpha)%*%t(JVector(h,alpha)),LowerPart1),
                           cbind(LowerPart1, JVector(h,alpha)%*%t(JVector(h,beta))-JVector(h,alpha)%*%t(JVector(h,alpha))))
      
      #Build the matrix to be exponentiated.
      Comega = rbind(cbind(Cphi,UpperPartomega),
                   cbind(LowerPart2, Cphi))
      MatrixExponentialomega = expm(Comega*te,method=expmethod)
      
      
      ExpQ = expm(Q*te,method=expmethod)
      
      #Now we can calculate the entries in the Hessian.
      #The formula for this is written down explicitly in the paper
      for(s in 1:(h-1)){
        for(r in 1:h){
          if(tmabs[s,r]>0){
            
            Hess[i,j] = Hess[i,j]+tmabs[s,r]*(-(1/Q[mu,nu]^2)*(ExpQ[s,r])^(-1)*(MatrixExponentialgamma[s,r+h])*(mu==alpha && nu==beta)
                                            -(1/Q[mu,nu])*(ExpQ[s,r])^(-2)*(MatrixExponentialeta[s,r+h])*(MatrixExponentialgamma[s,r+h]) 
                                            +(1/Q[mu,nu])*(ExpQ[s,r])^(-1)*(MatrixExponentialpsi[s,r+3*h])
                                            + (ExpQ[s,r])^(-2)*(MatrixExponentialeta[s,r+h])*(MatrixExponentialphi[s,r+h]) 
                                            -(ExpQ[s,r])^(-1)*(MatrixExponentialomega[s,r+3*h]))
          }
        }
      }
      
    }
  }
  
  
  #In order to find the information matrix we must take the negative of the
  #Hessian and invert it. So the information matrix is,
  
  CVmat= -solve((Hess+t(Hess))/2)
  #The estimates of the variance of q is the diagonal elements of Fisher
  #Recall for the normal distribution it is 1.96 standard deviations from the
  #mean.
  
  SEvec = sqrt(diag(CVmat))
  SEmat = matrix(0, nrow(tmabs), nrow(tmabs))
  for (k in 1:length(VQ[,1])) {
    SEmat[VQ[k,1], VQ[k,2]] = SEvec[k]
  }
  diagse = vector(length = nrow(tmabs))
  for (i in unique(VQ[,1])) {
    elem = VQ[which(VQ[,1] == i),2]
    if (length(elem) == 1) {
      diagse[i] = SEmat[i, elem]
    }
    else {
      combs = combn(elem, 2)
      CVsum = 0
      for (k in 1:ncol(combs)) {
        par1 = intersect(which(VQ[,1] == i), which(VQ[,2] == 
                                                     combs[1, k]))
        par2 = intersect(which(VQ[,1] == i), which(VQ[,2] == 
                                                     combs[2, k]))
        CVsum = CVsum + CVmat[par1, par2]
      }
      diagse[i] = sqrt(sum(SEmat[i, elem]^2) + 2 * CVsum)
    }
  }
  diag(SEmat) = diagse
  lowermat = Q - qnorm(1 - signif_level/2) * SEmat
  lowermat[which(SEmat == 0)] = NA
  lowermat[which(diag(Q) == 0), ] = 0
  uppermat = Q + qnorm(1 - signif_level/2) * SEmat
  uppermat[which(SEmat == 0)] = NA
  uppermat[which(diag(Q) == 0), ] = 0
  limits = list(lower = lowermat, upper = uppermat)
  limits
}

tmci=function (gmem, alpha, te, eps = 1e-04, expmethod = "PadeRBS") 
{
  JVector = function(h, i) {
    J = rep(0, h)
    J[i] = 1
    J
  }
  JMatrix = function(h, i, j) {
    J = matrix(0, h, h)
    J[i, j] = 1
    J
  }
  signif_level = alpha
  Q = gmem$par
  h = nrow(Q)
  te0 = gmem$te
  tmabs = gmem$tm
  QUseable = matrix(0, h, h)
  for (i in 1:h) {
    for (j in 1:h) {
      if (Q[i, j] > eps && j != i) 
        QUseable[i, j] = 1
    }
  }
  VQ = matrix(0, sum(sum(QUseable)), 2)
  Points = length(VQ[, 1])
  counter = 1
  for (i in 1:h) {
    for (j in 1:h) {
      if (QUseable[i, j] == 1) {
        VQ[counter, 1] = i
        VQ[counter, 2] = j
        counter = counter + 1
      }
    }
  }
  Hess = matrix(0, Points, Points)
  Jacobian = matrix(0, h^2, Points)
  LowerPart1 = matrix(0, h, h)
  LowerPart2 = matrix(0, 2 * h, 2 * h)
  ExpQ = expm(Q * te0, method = expmethod)
  Ceta = list()
  MatrixExponentialeta = list()
  MatrixExponentialetate = list()
  for (i in 1:Points) {
    alpha = VQ[i, 1]
    beta = VQ[i, 2]
    UpperParteta = JVector(h, alpha) %*% t(JVector(h, beta)) - 
      JVector(h, alpha) %*% t(JVector(h, alpha))
    Ceta[[i]] = rbind(cbind(Q, UpperParteta), cbind(LowerPart1, 
                                                    Q))
    MatrixExponentialeta[[i]] = expm(Ceta[[i]] * te0, method = expmethod)
    MatrixExponentialetate[[i]] = expm(Ceta[[i]] * te, method = expmethod)
  }
  for (i in 1:Points) {
    alpha = VQ[i, 1]
    beta = VQ[i, 2]
    for (j in 1:Points) {
      mu = VQ[j, 1]
      nu = VQ[j, 2]
      UpperPartxi = rbind(cbind(JVector(h, mu) %*% t(JVector(h, 
                                                             nu)) - JVector(h, mu) %*% t(JVector(h, mu)), 
                                LowerPart1), cbind(LowerPart1, JVector(h, mu) %*% 
                                                     t(JVector(h, nu)) - JVector(h, mu) %*% t(JVector(h, 
                                                                                                      mu))))
      Cxi = rbind(cbind(Ceta[[i]], UpperPartxi), cbind(LowerPart2, 
                                                       Ceta[[i]]))
      MatrixExponentialxi = expm(Cxi * te0, method = expmethod)
      for (s in 1:h) {
        for (r in 1:h) {
          Jacobian[(s - 1) * h + r, i] = MatrixExponentialetate[[i]][s, 
                                                                     h + r]
          if (tmabs[s, r] > 0) {
            Hess[i, j] = Hess[i, j] + tmabs[s, r] * ExpQ[s, 
                                                         r]^(-1) * (MatrixExponentialxi[s, r + 3 * 
                                                                                          h] - ExpQ[s, r]^(-1) * MatrixExponentialeta[[i]][s, 
                                                                                                                                           h + r] * MatrixExponentialeta[[j]][s, h + 
                                                                                                                                                                                r])
          }
        }
      }
    }
  }
  CVmat = solve(-Hess)
  SEmat = matrix(0, h, h)
  for (s0 in 1:h) {
    for (sT in 1:h) {
      SEmat[s0, sT] = t(Jacobian[(s0 - 1) * h + sT, ]) %*% 
        CVmat %*% t(t(Jacobian[(s0 - 1) * h + sT, ]))
    }
  }
  SEmat[which(diag(Q) == 0), ] = 0
  SEmat = sqrt(SEmat)
  P = expm(Q * te)
  lowermat = P - qnorm(1 - signif_level/2) * SEmat
  uppermat = P + qnorm(1 - signif_level/2) * SEmat
  limits = list(lower = lowermat, upper = uppermat, SE = SEmat)
  limits$method=paste0(te, " Period Delta Method Confidence Interval")
  limits$par=expm(gmem$par*te)
  limits$alpha=signif_level
  class(limits) = "gmci"
  limits
}

gmGS <-
function (tmabs, te, prior, burnin, conv_pvalue = 0, conv_freq = 10, 
    niter = 10000, sampl_method = "Unif", expmethod = "PadeRBS", 
    verbose = FALSE, combmat = NULL, sampl_func = NULL) 
{
    n = ncol(tmabs)
    gmest = matrix(0, n, n)
    for (i in 1:n) {
        for (j in setdiff(1:n, i)) {
            gmest[i, j] = rgamma(1, prior[[1]][i, j], rate = prior[[2]][i])
        }
    }
    diag(gmest) = -rowSums(gmest)
    gmestList = list()
    count = 0
    if(verbose==TRUE){
      pb=txtProgressBar(min=0,max=burnin+niter,title="Burn-in",style=3)
    }
    repeat {
        if (sampl_method == "Unif") {
            draw = rNijTRiT_Unif(tmabs, te, gmest, expm(gmest, 
                method = expmethod))
        }
        else if (sampl_method == "ModRej") {
            draw = rNijTRiT_ModRej(tmabs, te, gmest)
        }
        else if (sampl_method == "Comb") {
            Mmat = (combmat == "U") * tmabs
            Umat = (combmat == "M") * tmabs
            Mdraw = rNijTRiT_ModRej(Mmat, te, gmest)
            Udraw = rNijTRiT_Unif(Umat, te, gmest, expm(gmest, 
                method = expmethod))
            draw = list()
            draw$NijT = Mdraw$NijT + Udraw$NijT
            draw$RiT = Mdraw$RiT + Udraw$RiT
        }
        else if (sampl_method == "Ext") {
            draw = sampl_func(tmabs, te, gmest)
        }
        for (i in 1:n) {
            for (j in setdiff(1:n, i)) {
                gmest[i, j] = rgamma(1, draw$NijT[i, j] + prior[[1]][i, 
                  j], rate = (draw$RiT[i] + prior[[2]][i]))
            }
        }
        diag(gmest) = 0
        diag(gmest) = -rowSums(gmest)
        count = count + 1
        if (count > burnin) {
            gmestList[[length(gmestList) + 1]] = gmest
        }
        if (verbose == TRUE) setTxtProgressBar(pb,count)
        if (conv_pvalue>0 && count > burnin + 10 && (count - burnin)%%round(niter/conv_freq) == 
            0) {
            parmat = t(matrix(unlist(gmestList), n^2))
            parmat2 = parmat[, which(colSums(parmat) > 0)]
            mcmcobj = as.mcmc(parmat2)
            heidel = heidel.diag(mcmcobj, pvalue = conv_pvalue)
            if (verbose == TRUE) {
                print("Check Convergence")
            }
            if (sum(heidel[, 1]) == length(heidel[, 1]) & sum(heidel[, 
                4]) == length(heidel[, 4])) {
                if (verbose == TRUE) {
                  print("Converged according to Heidelberger and Welch Criterion")
                }
                break
            }
            if (verbose == TRUE) {
                print("Not Converged according to Heidelberger and Welch Criterion")
            }
        }
        if (count == (burnin + niter)) break
    }
    gmall = Reduce("+", gmestList)/(count - burnin)
    colnames(gmall) = colnames(tmabs)
    rownames(gmall) = rownames(tmabs)
    est = list(par = gmall, burnin = burnin, niter = count, draws = gmestList)
    return(est)
}

%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{A ctmcd Guide}


```{r setup, include=FALSE}
library(knitr)
```

# A ctmcd Guide

## Introduction

ctmcd is a package for the estimation of Markov generator matrices (i.e., the parameters of a continuous-time Markov chain) for the special case, that only discrete-time observations are given. The implemented methods to derive these estimates are diagonal adjustment ("DA", Israel et al., 2001), weighted adjustment ("WA", Israel et al., 2001), quasi-optimization ("QO", Kreinin and Sidelnikova, 2001), an instance of the expectation-maximization algorithm ("EM", Bladt and Soerensen, 2005) and a Gibbs Sampler ("GS", Bladt and Soerensen, 2005). For the expectation-maximization algorithm a Wald confidence interval according to the method of Oakes, 1999 can be derived. For the posterior mean estimate from the Gibbs sampler, an equal tailed credibility interval as outlined in Bladt and Soerensen, 2009 is implemented.

```{r}
library(ctmcd)
```

## Generator Matrix Estimation

In order to perform a generator matrix estimate, the discrete time data must be available as a matrix of either absolute ("EM", "GS") or relative transition frequencies ("DA", "WA", "QO"), depending on the method employed.

```{r}
data(tm_abs)
```
loads a credit transition example data set. In order to perform a diagonal adjustment generator matrix estimate, this data has to be converted into a matrix of relative transition frequencies first. Then, the gm method can be employed, requiring the time elapsed in the discrete time transition process, which is 1 as the example data has a single-year time horizon and furthermore a method specification, which is "DA" in this case.

```{r}
tm_rel=rbind((tm_abs/rowSums(tm_abs))[1:7,],c(rep(0,7),1))
gmda=gm(tm=tm_rel,te=1,method="DA")
gmda
```

A maximum likelihood estimate can be obtained by the EM algorithm, additionally requiring a starting value gmguess.
```{r}
gm0=matrix(1,8,8)
diag(gm0)=0
diag(gm0)=-rowSums(gm0)
gm0[8,]=0

gmem=gm(tm_abs,te=1,method="EM",gmguess=gm0)
gmem
```


## Confidence / Credibility Intervals
Interval estimates of gm objects can only be obtained for the methods "EM" and "GS" by means of the gmci function, e.g., a 95% Wald confidence interval can be obtained by

```{r}
ciem=gmci(gmem,alpha=0.05)
```

## Matrix Plot Function
Both, gm and gmci objects can be visualized by the matrix plot function plotM(), which can be easily accessed by the wrapper function plot().

```{r}
plot(gmem)
```

## References
M. Bladt and M. Soerensen: _Statistical Inference for Discretely Observed Markov Jump Processes._ Journal of the Royal Statistical Society B 67(3):395-410, 2005

M. Bladt and M. Soerensen. _Efficient Estimation of Transition Rates Between Credit Ratings from Observations at Discrete Time Points._ Quantitative Finance, 9(2):147-160, 2009

R. B. Israel et al.: _Finding Generators for Markov Chains via Empirical Transition Matrices, with Applications to Credit Ratings._ Mathematical Finance 11(2):245-265, 2001

E. Kreinin and M. Sidelnikova: _Regularization Algorithms for Transition Matrices._ Algo Research Quarterly 4(1):23-40, 2001

M. Pfeuffer: _ctmcd: An R Package for Estimating the Parameters of a Continuous-Time Markov Chain from Discrete-Time Data._ The R Journal 9(2):127-141, 2017

D. Oakes. _Direct calculation of the information matrix via the EM algorithm._ Journal of the Royal Statistical Society: Series B (Statistical Methodology), 61(2):479-482, 1999

\name{ctmcd-package}
\alias{ctmcd-package}
\alias{ctmcd}
\docType{package}
\title{
\packageTitle{ctmcd}
}
\description{
Functions for estimating Markov generator matrices from discrete-time observations.
}
\author{
\packageAuthor{ctmcd}

Maintainer: \packageMaintainer{ctmcd}
}
\references{
M. Pfeuffer: ctmcd: An R Package for Estimating the Parameters of a Continuous-Time Markov Chain from Discrete-Time Data. The R Journal 9(2):127-141, 2017

M. Pfeuffer. Generator Matrix Approximation Based on Discrete-Time Rating Migration Data. Master
Thesis, Ludwig Maximilian University of Munich, 2016

R. B. Israel et al.: Finding Generators for Markov Chains via Empirical Transition Matrices, with Applications to Credit Ratings. Mathematical Finance 11(2):245-265, 2001

E. Kreinin and M. Sidelnikova: Regularization Algorithms for Transition Matrices. Algo Research Quarterly 4(1):23-40, 2001

M. Bladt and M. Soerensen: Statistical Inference for Discretely Observed Markov Jump Processes. Journal of the Royal Statistical Society B 67(3):395-410, 2005

}

\keyword{ package }

\examples{
\dontrun{
data(tm_abs)

## Maximum Likelihood Generator Matrix Estimate
gm0=matrix(1,8,8)
diag(gm0)=0
diag(gm0)=-rowSums(gm0)
gm0[8,]=0

gmem=gm(tm_abs,te=1,method="EM",gmguess=gm0)
plot(gmem)

## Confidence Interval
ciem=gmci(gmem,alpha=0.05)
plot(ciem)
}
}

\name{summary.gm}
\alias{summary.gm}

\title{
Extended Output for Generator Matrix Estimate Objects
}
\description{
Function for providing results and extended output of a generator matrix estimation procedure.
}
\usage{
\method{summary}{gm}(object, ...)
}

\arguments{
  \item{object}{
a generator matrix estimation object
}
  \item{\dots}{
additional arguments
}
}
\seealso{
\code{\link{print.gm}}, \code{\link{plot.gm}}
}

\name{print.gm}
\alias{print.gm}

\title{
Print Method for Generator Matrix Estimation Objects
}
\description{
Function for printing the results of a generator matrix estimation}
\usage{
\method{print}{gm}(x, ...)
}

\arguments{
  \item{x}{
a generator matrix estimation object
}
  \item{\dots}{
additional arguments
}
}

\seealso{
\code{\link{summary.gm}}, \code{\link{plot.gm}}
}

\name{plotM}
\alias{plotM}
\title{
Matrix Plot Function
}
\description{
Function to visualize matrices
}
\usage{
plotM(mat, mattext, col = c("grey", "red"), main, las = 1, xlab = "To", 
ylab = "From", xnames, ynames, cex = min(1, nrow(mat)/8), fig = 3, opacity_factor)
}
\arguments{
  \item{mat}{
a matrix
}
  \item{mattext}{
optional: matrix of strings replacing the original matrix entries
}
  \item{col}{
two element vector of basis colors for positive and negative matrix entries
}
  \item{main}{
optional: plot title
}
  \item{las}{
orientation of x and y axis elements
}
  \item{xlab}{
x axis name
}
  \item{ylab}{
y axis name
}
  \item{xnames}{
description of x axis elements
}
  \item{ynames}{
description of y axis elements
}
  \item{cex}{
font size
}
  \item{fig}{
number of significant figures to be plotted
}
  \item{opacity_factor}{
two element vector for specification of opacity for positive and negative parameter entry highlighting (must be greater than zero)
}
}
\author{
Marius Pfeuffer
}\seealso{
\code{\link{plot.gm}}, \code{\link{plot.gmci}}
}
\examples{gm0=matrix(1,8,8)
diag(gm0)=0
diag(gm0)=-rowSums(gm0)
gm0[8,]=0

plotM(gm0)
}

\name{gmWA}
\alias{gmWA}
\title{
Weighted Adjustment
}
\description{
Function for deriving a Markov generator matrix estimate based on the weighted adjustment method of Israel et al., 2001
}
\usage{
gmWA(tmrel, te, logmethod = "Eigen")
}
\arguments{
  \item{tmrel}{
matrix of relative transition frequencies
}
  \item{te}{
time elapsed in transition process
}
  \item{logmethod}{
method for computation of matrix logarithm, by default eigendecomposition is chosen (see \code{?logm} from \code{expm} package for more information)
}
}

\details{
A candidate solution is derived by the matrix logarithm and then adjusted in order to fulfil the properties of a Markov generator matrix.
}


\references{
R. B. Israel et al.: Finding Generators for Markov Chains via Empirical Transition Matrices, with Applications to Credit Ratings. Mathematical Finance 11(2):245-265, 2001
}

\author{
Marius Pfeuffer
}

\examples{
## Derive matrix of relative transition frequencies
data(tm_abs)
tm_rel=rbind((tm_abs/rowSums(tm_abs))[1:7,],c(rep(0,7),1))

## Derive weighted adjustment generator matrix estimate
gmwa=gmWA(tm_rel,1)
gmwa
}

\name{gmci}
\alias{gmci}
\title{
Confidence / Credibility Intervals for Generator Matrix Objects
}
\description{
Generic function to derive confidence / credibility intervals for "EM" or "GS" based generator matrix objects
}
\usage{
gmci(gm, alpha, ...)
}
\arguments{
  \item{gm}{
a "EM" or "GS" generator matrix object
}
  \item{alpha}{
significance level
}
\item{...}{additional arguments:
\itemize{
  \item{eps:}{
threshold for which generator matrix parameters are assumed to be fixed at zero (if "EM" object)
}
\item{cimethod:}{
"Direct" and "SdR" use analytical expressions of the Fisher information matrix, "BS" employs the numerical approach of Bladt and Soerensen, 2009 (if "EM" object)
}
  \item{expmethod:}{
method to compute matrix exponentials (see \code{?expm} from \code{expm} package for more information)
}
}
}
}
\details{
If gm is based on the "EM" method (expectation-maximization algorithm), the function computes a Wald confidence interval based on the method of Oakes, 1999. IF gm is based on the "GS" method (Gibbs sampler), the function computes an equal-tailed credibility interval.
}

\references{

M. Bladt and M. Soerensen. Efficient Estimation of Transition Rates Between Credit Ratings from Observations at Discrete Time Points. Quantitative Finance, 9(2):147-160, 2009

D. Oakes. Direct calculation of the information matrix via the EM algorithm. Journal of the Royal Statistical Society: Series B (Statistical Methodology), 61(2):479-482, 1999

G. Smith and G. dos Reis. Robust and Consistent Estimation of Generators in Credit Risk. Quantitative Finance 18(6):983-1001, 2018

G. dos Reis, M. Pfeuffer, G. Smith: Capturing Rating Momentum in the Estimation of Probabilities of Default, With Application to Credit Rating Migrations (In Preparation), 2018
}
\author{
Marius Pfeuffer
}


\examples{
\dontrun{
data(tm_abs)

## Maximum Likelihood Generator Matrix Estimate
gm0=matrix(1,8,8)
diag(gm0)=0
diag(gm0)=-rowSums(gm0)
gm0[8,]=0

gmem=gm(tm_abs,te=1,method="EM",gmguess=gm0)

## Oakes Confidence Interval
ciem=gmci(gmem,alpha=0.05)
ciem
}
}

\name{rNijTRiT_Unif}
\alias{rNijTRiT_Unif}
\title{
C++ Based Uniformization Sampling
}
\description{
Function for generating initial and endpoint-conditioned Markov process sampling paths for a given discrete-time transition matrix
}
\usage{
rNijTRiT_Unif(tmabs, te, gm, tpm) 
}

\arguments{
  \item{tmabs}{
matrix of absolute transition frequencies
}
  \item{te}{
time elapsed in transition process
}
  \item{gm}{
generator matrix
}
  \item{tpm}{
discrete-time transition probability matrix, matrix exponential of gm
} 
}
\details{
Function for the simulation of paths from an endpoint-conditioned Markov process. Returns number of transitions NijT and cumulative holding times RiT.
}

\value{
endpoint-conditioned sampling path
}

\references{
J. Fintzi: R Package ECctmc, 2016.

A. Hobolth and E. A. Stone: Simulation from Endpoint-Conditioned, Continuous-Time Markov Chains on a Finite State Space, with Applications to Molecular Evolution. Annals of Applied Statistics 3(3):1204-1231, 2009
}
\author{
Jon Fintzi, Marius Pfeuffer
}
\examples{
data(tm_abs)

## Generator Matrix
gm=matrix(1,8,8)
diag(gm)=0
diag(gm)=-rowSums(gm)
gm[8,]=0

## Transition Probability Matrix
library(expm)
te=1
tpm=expm(gm*te)

rNijTRiT_Unif(tm_abs,te,gm,tpm)
}

\name{gmGS}
\alias{gmGS}

\title{
Gibbs Sampler
}
\description{
Function for deriving a Markov generator matrix estimate by Gibbs sampling (described by Bladt and Soerensen, 2005)
}
\usage{
gmGS(tmabs, te, prior, burnin, conv_pvalue = 0, conv_freq = 10,
niter = 10000, sampl_method = "Unif", expmethod = "PadeRBS", verbose = FALSE, 
combmat=NULL, sampl_func = NULL)
}
\arguments{
  \item{tmabs}{
matrix of absolute transition frequencies
}
  \item{te}{
time elapsed in transition process
}
  \item{prior}{
list of prior parameters (Gamma prior)
}
  \item{burnin}{
number of burn-in iterations
}
  \item{conv_pvalue}{
convergence criterion: stop, if Heidelberger and Welch's diagnostic assumes convergence (see coda package), convergence check is only employed if conv_pvalue>0
}
 \item{conv_freq}{
convergence criterion: absolute frequency of convergence evaluations
}
  \item{niter}{
stop criterion: stop, if maximum number of iterations is exceeded
}
  \item{sampl_method}{
method for sampling paths from endpoint-conditioned Markov processes. options: "Unif" - Uniformization sampling, "ModRej" - Modified Rejection Sampling
}
  \item{expmethod}{
method for computation of matrix exponential, by default "PadeRBS" is chosen (see \code{?expm} from \code{expm} package for more information)
}
  \item{verbose}{
verbose mode
}
  \item{combmat}{
matrix specifying the combined use of sampling methods: "U" - uniformization sampling, "M" - modified rejection sampling
}
  \item{sampl_func}{
interface for own endpoint-conditioned Markov process sampling function
}
}
\details{
A posterior mean generator matrix estimate is derived by Gibbs Sampling. The gamma distribution is used as prior.
}


\value{
generator matrix estimate
}
 
\references{
M. Bladt and M. Soerensen: Statistical Inference for Discretely Observed Markov Jump Processes. Journal of the Royal Statistical Society B 67(3):395-410, 2005


}
\author{
Marius Pfeuffer
}

\seealso{
\code{\link{rNijTRiT_ModRej}}, \code{\link{rNijTRiT_Unif}}
}

\examples{
data(tm_abs)

## Example prior parametrization (absorbing default state)
pr=list()
pr[[1]]=matrix(1,8,8)
pr[[1]][8,]=0

pr[[2]]=c(rep(5,7),Inf)

## Derive Gibbs sampling generator matrix estimate
\donttest{
gmgs=gmGS(tmabs=tm_abs,te=1,sampl_method="Unif",prior=pr,burnin=10,niter=100,verbose=TRUE)
gmgs
}

}

\name{plot.gm}
\alias{plot.gm}

\title{
Plot Function for Generator Matrix Estimation Objects
}
\description{
Function for visualizing the output of a generator matrix estimation procedure.
}
\usage{
\method{plot}{gm}(x, mattext, col = c("grey", "red"), main = x$method, las = 1, 
xlab = "To", ylab = "From", xnames, ynames, cex = 1, fig = 3, opacity_factor, ...)
}
\arguments{
  \item{x}{
a generator matrix estimation object
}
  \item{mattext}{
optional: matrix of strings replacing the parameter estimates
}
  \item{col}{
two element vector of basis colors for positive and negative parameter estimate entries
}
  \item{main}{
optional: plot title
}
  \item{las}{
orientation of x and y axis elements
}
  \item{xlab}{
x axis name
}
  \item{ylab}{
y axis name
}
  \item{xnames}{
description of x axis elements
}
  \item{ynames}{
description of y axis elements
}
  \item{cex}{
font size
}
  \item{fig}{
number of significant figure to be plotted
}
  \item{opacity_factor}{
two element vector for specification of opacity for positive and negative parameter entry highlighting (must be greater than zero)
}
  \item{\dots}{
additional arguments
}
}

\author{
Marius Pfeuffer
}

\seealso{
\code{\link{print.gm}}, \code{\link{summary.gm}}, \code{\link{plotM}}
}
\examples{
data(tm_abs)

## Maximum Likelihood Generator Matrix Estimate
gm0=matrix(1,8,8)
diag(gm0)=0
diag(gm0)=-rowSums(gm0)
gm0[8,]=0

gmem=gm(tm_abs,te=1,method="EM",gmguess=gm0)
plot(gmem)
}

\name{ctmcdlogLik}
\alias{ctmcdlogLik}
\title{
Discrete-Time Data Log-Likelihood Function
}
\description{
Function for evaluating the likelihood function of a continuous-time Markov chain given discrete-time data.
}
\usage{
ctmcdlogLik(gm, tmabs, te)
}

\arguments{
\item{gm}{
generator matrix of continuous-time Markov chain
}
  \item{tmabs}{
matrix of absolute transition frequencies
}
  \item{te}{
time elapsed in transition process
}
 
}
\value{
log-likelihood value
}
\author{
Marius Pfeuffer
}
\examples{
data(tm_abs)

## Initial guess for generator matrix (absorbing default state)
gm0=matrix(1,8,8)
diag(gm0)=0
diag(gm0)=-rowSums(gm0)
gm0[8,]=0

## Log-likelihood of initial guess
ctmcdlogLik(gm0,tm_abs,1)
}

\name{tmci}
\alias{tmci}
\title{
Delta Method Confidence Intervals for Matrix Exponential Transformations of Generator Matrix Objects
}
\description{
Generic function to derive delta method based confidence intervals for matrix exponential transformations of "EM" based generator matrix objects
}
\usage{
tmci(gmem, alpha, te, eps = 1e-04, expmethod = "PadeRBS")
}
\arguments{
  \item{gmem}{
an "EM" generator matrix object
}
  \item{alpha}{
significance level
}
  \item{te}{
discrete time horizon for which the interval is supposed to be computed
}
 \item{eps}{
threshold for which generator matrix parameters are assumed to be fixed at zero
}
  \item{expmethod}{
method to compute matrix exponentials (see \code{?expm} from \code{expm} package for more information)
}
}
\details{
Confidence intervals for discrete-time transition matrix predictions given generator matrix estimates are computed by using the delta method for matrix exponential transformations. 
}

\references{
G. dos Reis, M. Pfeuffer, G. Smith: Capturing Rating Momentum in the Estimation of Probabilities of Default, With Application to Credit Rating Migrations (In Preparation), 2018
}

\examples{
\dontrun{
data(tm_abs)

## Maximum Likelihood Generator Matrix Estimate
gm0=matrix(1,8,8)
diag(gm0)=0
diag(gm0)=-rowSums(gm0)
gm0[8,]=0

gmem=gm(tm_abs,te=1,method="EM",gmguess=gm0)

## 2.5 Year Transition Matrix Confidence Interval
citm=tmci(gmem,alpha=0.05,te=2.5)
citm
}
}

\name{gm.default}
\alias{gm.default}

\title{
Generator Matrix Estimation
}
\description{
Default function to estimate the parameters of a continuous Markov chain
}
\usage{
\method{gm}{default}(tm, te, method, gmguess = NULL, prior = NULL, burnin = NULL, 
eps = 1e-06, conv_pvalue = 0.05, conv_freq = 10, niter = 10000, sampl_func = NULL, 
combmat = NULL, sampl_method = "Unif", logmethod = "Eigen", expmethod = "PadeRBS", 
verbose = FALSE, ...)
}

\arguments{
  \item{tm}{
matrix of either absolute transition frequencies (if method is "EM" or "GS") or relative transition frequencies (if method is "DA", "WA" of "QO")
}
  \item{te}{
time elapsed in transition process
}
  \item{method}{
method to derive generator matrix: "DA" - Diagonal Adjustment, "WA" - Weighted Adjustment, "QO" - Quasi-Optimization, "EM" - Expectation-Maximization Algorithm, "GS" - Gibbs Sampler
}
  \item{gmguess}{
initial guess for generator matrix estimation procedure (if method is "EM")
}
  \item{prior}{
prior parametrization (if method is "GS")
}
  \item{burnin}{
burn-in period (if method is "GS")
}
  \item{eps}{
convergence criterion (if method is "EM" or "GS")
}
  \item{conv_pvalue}{
convergence criterion: stop, if Heidelberger and Welch's diagnostic assumes convergence (see coda package)
}
 \item{conv_freq}{
convergence criterion: absolute frequency of convergence evaluations
}
  \item{niter}{
maximum number of iterations (if method is "EM" or "GS")
}
  \item{sampl_func}{
optional self-written path sampling function for endpoint-conditioned Markov processes (if method is "GS")
}
  \item{combmat}{
matrix stating combined use of modified rejection sampling / uniformization sampling algorithms (if method is "GS")
}
  \item{sampl_method}{
sampling method for deriving endpoint-conditioned Markov process path: "Unif" - Uniformization Sampling, "ModRej" - Modified Rejection Sampling (if method is "GS")
}
  \item{logmethod}{
method to compute matrix logarithm (if method is "DA", "WA" or "QO", see \code{?logm} from \code{expm} package for more information)
}
  \item{expmethod}{
method to compute matrix exponential (if method is "EM" or "GS", see \code{?expm} from \code{expm} package for more information)
}
  \item{verbose}{
verbose mode (if method is "EM" or "GS")
}
  \item{\dots}{
additional arguments
}
}
\details{
The methods "DA", "WA" and "QO" provide adjustments of a matrix logarithm based candidate solution, "EM" gives the maximum likelihood estimate and "GS" a posterior mean estimate in a Bayesian setting with conjugate Gamma priors.
}


\references{
M. Pfeuffer: Generator Matrix Approximation Based on Discrete Time Rating Migration Data. Master Thesis, University of Munich, 2016

Y. Inamura: Estimating Continuous Time Transition Matrices from Discretely Observed Data. Bank of Japan Working Paper Series, 2006

R. B. Israel et al.: Finding Generators for Markov Chains via Empirical Transition Matrices, with Applications to Credit Ratings. Mathematical Finance 11(2):245-265, 2001

E. Kreinin and M. Sidelnikova: Regularization Algorithms for Transition Matrices. Algo Research Quarterly 4(1):23-40, 2001

M. Bladt and M. Soerensen: Statistical Inference for Discretely Observed Markov Jump Processes. Journal of the Royal Statistical Society B 67(3):395-410, 2005

}
\author{
Marius Pfeuffer
}


\seealso{
\code{\link{gmDA}}, \code{\link{gmWA}}, \code{\link{gmQO}}, \code{\link{gmEM}}, \code{\link{gmGS}}
}

\examples{
data(tm_abs)

## Maximum Likelihood Generator Matrix Estimate
gm0=matrix(1,8,8)
diag(gm0)=0
diag(gm0)=-rowSums(gm0)
gm0[8,]=0

gmem=gm(tm_abs,te=1,method="EM",gmguess=gm0)
gmem

## Quasi Optimization Estimate
tm_rel=rbind((tm_abs/rowSums(tm_abs))[1:7,],c(rep(0,7),1))

gmqo=gm(tm_rel,te=1,method="QO")
gmqo
}

\name{tm_abs}
\alias{tm_abs}
\docType{data}
\title{
Single Year Corporate Credit Rating Transititions
}
\description{
Matrix of Standard and Poor's Global Corporate Rating Transition Frequencies 2000 (NR Removed)
}
\usage{data("tm_abs")}
\format{
  The format is:
 num [1:8, 1:8] 17 2 0 0 0 0 0 0 1 455 ...
 - attr(*, "dimnames")=List of 2
  ..$ : chr [1:8] "AAA" "AA" "A" "BBB" ...
  ..$ : chr [1:8] "AAA" "AA" "A" "BBB" ...
}

\references{
European Securities and Markets Authority, 2016 

https://cerep.esma.europa.eu/cerep-web/statistics/transitionMatrice.xhtml
}
\examples{
data(tm_abs)

## Matrix of relative transition frequencies
tm_rel=rbind((tm_abs/rowSums(tm_abs))[1:7,],c(rep(0,7),1))
tm_rel

}
\keyword{datasets}

\name{rNijTRiT_ModRej}
\alias{rNijTRiT_ModRej}
\title{
C++ Based Modified Rejection Sampling
}
\description{
Function for generating initial and endpoint-conditioned Markov process sampling paths for a given discrete-time transition matrix

}
\usage{
rNijTRiT_ModRej(tmabs, te, gm) 
}

\arguments{
  \item{tmabs}{
matrix of absolute transition frequencies
}
  \item{te}{
time elapsed in transition process
}
  \item{gm}{
generator matrix
}
}
\details{
Function for the simulation of paths from an endpoint-conditioned Markov process. Returns number of transitions NijT and cumulative holding times RiT.
}
\references{
J. Fintzi: R Package ECctmc, 2016.

A. Hobolth and E. A. Stone: Simulation from Endpoint-Conditioned, Continuous-Time Markov Chains on a Finite State Space, with Applications to Molecular Evolution. Annals of Applied Statistics 3(3):1204-1231, 2009
}
\author{
Jon Fintzi, Marius Pfeuffer
}
\examples{
data(tm_abs)

## Initial guess for generator matrix (absorbing default state)

gm=matrix(1,8,8)
diag(gm)=0
diag(gm)=-rowSums(gm)
gm[8,]=0

rNijTRiT_ModRej(tm_abs,1,gm)
}

\name{gmEM}
\alias{gmEM}
\title{
Expectation-Maximization Algorithm
}
\description{
Function for deriving a Markov generator matrix estimate by an instance of the expectation-maximization algorithm (described by Bladt and Soerensen, 2005)

}
\usage{
gmEM(tmabs, te, gmguess, eps = 1e-06, niter = 10000, expmethod = "PadeRBS", 
verbose = FALSE)
}

\arguments{
  \item{tmabs}{
matrix of absolute transition frequencies
}
  \item{te}{
time elapsed in transition process
}
  \item{gmguess}{
initial guess (for generator matrix)
}
  \item{eps}{
stop criterion: stop, if relative change in log-likelihood is smaller than eps
}
  \item{niter}{
stop criterion: maximum number of iterations
}
  \item{expmethod}{
method for computation of matrix exponential, by default "PadeRBS" is chosen (see \code{?expm} from \code{expm} package for more information)
}
  \item{verbose}{
verbose mode
}
}
\details{
A maximum likelihood generator matrix estimate is derived by an instance of the expectation-maximization algorithm. 
}

\value{
generator matrix estimate
}
\references{
M. Bladt and M. Soerensen: Statistical Inference for Discretely Observed Markov Jump Processes. Journal of the Royal Statistical Society B 67(3):395-410, 2005


}
\author{
Marius Pfeuffer
}
\examples{
data(tm_abs)

## Initial guess for generator matrix (absorbing default state)
gm0=matrix(1,8,8)
diag(gm0)=0
diag(gm0)=-rowSums(gm0)
gm0[8,]=0

## Derive expectation-maximization algorithm generator matrix estimate
gmem=gmEM(tmabs=tm_abs,1,gmguess=gm0,verbose=TRUE)
gmem
}

\name{gmQO}
\alias{gmQO}
\title{
Quasi-Optimization
}
\description{
Function for deriving a Markov generator matrix estimate based on the quasi-optimization procedure of Kreinin and Sidelnikova, 2001
}
\usage{
gmQO(tmrel, te, logmethod = "Eigen")
}
\arguments{
  \item{tmrel}{
matrix of relative transition frequencies
}
  \item{te}{
time elapsed in transition process
}
  \item{logmethod}{
method for computation of matrix logarithm, by default eigendecomposition is chosen (see \code{?logm} from \code{expm} package for more information)
}
}
\details{
From the set of possible Markov generator matrices, the one is chosen which is closest to a matrix logarithm based candidate solution in terms of sum of squared deviations.
}

\value{
generator matrix estimate
}
\references{
E. Kreinin and M. Sidelnikova: Regularization Algorithms for Transition Matrices. Algo Research Quarterly 4(1):23-40, 2001
}
\author{
Marius Pfeuffer
}
\examples{
data(tm_abs)
## Derive matrix of relative transition frequencies
data(tm_abs)
tm_rel=rbind((tm_abs/rowSums(tm_abs))[1:7,],c(rep(0,7),1))

## Derive quasi optimization generator matrix estimate
gmqo=gmQO(tm_rel,1)
gmqo
}

\name{gm}
\alias{gm}

\title{
Generator Matrix Estimation
}
\description{
Generic function to estimate the parameters of a continuous Markov chain
}
\usage{
gm(tm, te, method, ...)
}

\arguments{
  \item{tm}{
matrix of either absolute transition frequencies (if method is "EM" or "GS") or relative transition frequencies (if method is "DA", "WA" of "QO")
}
  \item{te}{
time elapsed in transition process
}
  \item{method}{
method to derive generator matrix: "DA" - Diagonal Adjustment, "WA" - Weighted Adjustment, "QO" - Quasi-Optimization, "EM" - Expectation-Maximization Algorithm, "GS" - Gibbs Sampler
}
 \item{...}{Additional Arguments:
 \itemize{
  \item{gmguess:}{
initial guess for generator matrix estimation procedure (if method is "EM")
}
  \item{prior:}{
prior parametrization (if method is "GS")
}
  \item{burnin:}{
burn-in period (if method is "GS")
}
  \item{eps:}{
convergence criterion (if method is "EM")
}
  \item{conv_pvalue,conv_freq:}{
convergence criterion (if method is "GS")
  }
  \item{niter:}{
maximum number of iterations (if method is "EM" or "GS")
}
  \item{sampl_func:}{
optional self-written path sampling function for endpoint-conditioned Markov processes (if method is "GS")
}
  \item{combmat:}{
matrix stating combined use of modified rejection sampling / uniformization sampling algorithms (if method is "GS")
}
  \item{sampl_method:}{
sampling method for deriving endpoint-conditioned Markov process path: "Unif" - Uniformization Sampling, "ModRej" - Modified Rejection Sampling (if method is "GS")
}
  \item{logmethod:}{
method to compute matrix logarithm (if method is "DA", "WA" or "QO", see \code{?logm} from \code{expm} package for more information)
}
  \item{expmethod:}{
method to compute matrix exponential (if method is "EM" or "GS", see \code{?expm} from \code{expm} package for more information)
}
  \item{verbose:}{
verbose mode (if method is "EM" or "GS")
}
}
}
}
\details{
The methods "DA", "WA" and "QO" provide adjustments of a matrix logarithm based candidate solution, "EM" gives the maximum likelihood estimate and "GS" a posterior mean estimate in a Bayesian setting with conjugate Gamma priors.
}


\references{

G. dos Reis, M. Pfeuffer, G. Smith: Capturing Rating Momentum in the Estimation of Probabilities of Default, With Application to Credit Rating Migrations (In Preparation), 2018

M. Pfeuffer: Generator Matrix Approximation Based on Discrete Time Rating Migration Data. Master Thesis, University of Munich, 2016

Y. Inamura: Estimating Continuous Time Transition Matrices from Discretely Observed Data. Bank of Japan Working Paper Series, 2006

R. B. Israel et al.: Finding Generators for Markov Chains via Empirical Transition Matrices, with Applications to Credit Ratings. Mathematical Finance 11(2):245-265, 2001

E. Kreinin and M. Sidelnikova: Regularization Algorithms for Transition Matrices. Algo Research Quarterly 4(1):23-40, 2001

M. Bladt and M. Soerensen: Statistical Inference for Discretely Observed Markov Jump Processes. Journal of the Royal Statistical Society B 67(3):395-410, 2005

}
\author{
Marius Pfeuffer
}


\seealso{
\code{\link{gmDA}}, \code{\link{gmWA}}, \code{\link{gmQO}}, \code{\link{gmEM}}, \code{\link{gmGS}}
}

\examples{
data(tm_abs)

## Maximum Likelihood Generator Matrix Estimate
gm0=matrix(1,8,8)
diag(gm0)=0
diag(gm0)=-rowSums(gm0)
gm0[8,]=0

gmem=gm(tm_abs,te=1,method="EM",gmguess=gm0)
gmem

## Quasi Optimization Estimate
tm_rel=rbind((tm_abs/rowSums(tm_abs))[1:7,],c(rep(0,7),1))

gmqo=gm(tm_rel,te=1,method="QO")
gmqo
}

\name{gmci.default}
\alias{gmci.default}
\alias{ciEMBS}
\alias{ciEMSdR}
\alias{ciEMoFI}
\alias{ciEMDM}
\alias{ciGS}
\title{
Confidence / Credibility Intervals for Generator Matrix Objects
}
\description{
Default function to derive confidence / credibility intervals for "EM" or "GS" based generator matrix objects
}
\usage{
\method{gmci}{default}(gm, alpha, eps = 1e-04, cimethod="Direct", expmethod = "PadeRBS", ...)
}
\arguments{
  \item{gm}{
a "EM" or "GS" generator matrix object
}
  \item{alpha}{
significance level
}
  \item{eps}{
threshold for which generator matrix parameters are assumed to be fixed at zero (if "EM" object)
}
 \item{cimethod}{
"Direct" or "SdR" use analytical expressions of the Fisher information matrix, "BS" emloy the numerical expressions of Bladt and Soerensen, 2009 (if "EM" object)
}
  \item{expmethod}{
method to compute matrix exponentials (see \code{?expm} from \code{expm} package for more information)
}
  \item{\dots}{
additional arguments
}
}
\details{
If gm is based on the "EM" method (expectation-maximization algorithm), the function computes a Wald confidence interval based on the method of Oakes, 1999. IF gm is based on the "GS" method (Gibbs sampler), the function computes an equal-tailed credibility interval.
}

\references{

G. dos Reis, M. Pfeuffer, G. Smith: Capturing Rating Momentum in the Estimation of Probabilities of Default, With Application to Credit Rating Migrations (In Preparation), 2018
}

\author{
Marius Pfeuffer
}


\examples{
\dontrun{
data(tm_abs)

## Maximum Likelihood Generator Matrix Estimate
gm0=matrix(1,8,8)
diag(gm0)=0
diag(gm0)=-rowSums(gm0)
gm0[8,]=0

gmem=gm(tm_abs,te=1,method="EM",gmguess=gm0)

## Oakes Confidence Interval
ciem=gmci(gmem,alpha=0.05)
ciem
}
}

\name{expmMC}
\alias{expmMC}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Matrix Exponential Function
}
\description{
Computation of the matrix exponential and interface to the markovchain package
}
\usage{
expmMC(gm, t, method = "PadeRBS", order = 8)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{gm}{
generator matrix (either gm or matrix object)
}
  \item{t}{
time horizon
}
  \item{method}{
method to compute the matrix exponential, see expm for details
}
  \item{order}{
order for Pade or Taylor method, see expm for details
}
}
\details{
An interface to the markovchain package is provided so that the resulting transition matrices are returned as markovchain objects and can be further processed in the markovchain package, e.g., visualized (see example below).
}

\references{
G. A. Spedicato: Discrete Time Markov Chains with R. The R Journal (To Appear), 2017
}
\author{
G. A. Spedicato, M. Pfeuffer
}


%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{expm}}
}
\examples{
### Exemplary Transition Matrix
tm0=matrix(1:16,4,4)
tm0=tm0/rowSums(tm0)
tm0[4,]=c(0,0,0,1)

### Generator Matrix Estimate
gm_est=gm(tm0,te=1,method="DA")
gm_est

### Matrix Exponential and Conversion to markovchain object 
### (markovchain package needs to be installed)
mc=expmMC(gm_est,.5)
if(require("markovchain")==TRUE){
  plot(mc)
}
}

\name{print.gmci}
\alias{print.gmci}
\title{
Print Method for Generator Matrix Confidence / Credibility Interval Objects
}
\description{
Function for printing the boundaries of a generator matrix confidence / credibility interval}
\usage{
\method{print}{gmci}(x, ...)
}
\arguments{
  \item{x}{
a generator matrix confidence / credibility interval
}
  \item{\dots}{
additional arguments
}
}\seealso{
\code{\link{plot.gmci}}
}
